package com.huyi.service.conmany.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huyi.service.base.entity.SysCompany;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 企业表 Mapper 接口
 * </p>
 *
 * @author songqg
 * @since 2021-08-16
 */
public interface OwnCompanyManageMapper extends BaseMapper<SysCompany> {

    /**
     * 查询发票信息
     * @param map
     * @return
     */
    public List<Map> selectInvoiceList(@Param("paramMap")Map map);

    /**
     * 查询用户信息
     * @param map
     * @return
     */
    public List<Map> selectUserList(@Param("paramMap")Map map);

    /**
     * 查询营业执照信息
     * @param map
     * @return
     */
    public List<Map> selectLicenseFileList(@Param("paramMap")Map map);

    /**
     * 查询法人证件正面信息
     * @param map
     * @return
     */
    public List<Map> selectLegalFileList(@Param("paramMap")Map map);

    /**
     * 查询法人证件反面信息
     * @param map
     * @return
     */
    public List<Map> selectLegalBackFileList(@Param("paramMap")Map map);

    /**
     * 查询法人授权书信息
     * @param map
     * @return
     */
    public List<Map> selectAuthorizationFileList(@Param("paramMap")Map map);

    /**
     * 查询数字证书授权与承诺书信息
     * @param map
     * @return
     */
    public List<Map> selectCommitmentFileList(@Param("paramMap")Map map);

    /**
     * 查询其他附件信息
     * @param map
     * @return
     */
    public List<Map> selectAppendixFileList(@Param("paramMap")Map map);

    /**
     * 经办人证件正面信息
     * @param map
     * @return
     */
    public List<Map> handlerPositiveList(@Param("paramMap")Map map);

    /**
     * 经办人证件反面信息
     * @param map
     * @return
     */
    public List<Map> handlerNegativeList(@Param("paramMap")Map map);

}
